#!/bin/bash -x

if [ -z "$IDA_PRO_SERVER_HOST" ]; then echo Failed to set IDA_PRO_SERVER_HOST; exit 2; fi
if [ -z "$IDA_PRO_SERVER_USER" ]; then
    IDA_PRO_SERVER_USER=`whoami`
fi
if [ -z "$IDA_PRO_SERVER_PORT" ]; then
    IDA_PRO_SERVER_PORT=22
fi

file=$1
md5name=$(md5sum $file | awk '{print $1}')
directory=/tmp/remote-analyze/${md5name}

verify_host_live()
{
	remotehost=$1
	ping -c 5 -w 5 $remotehost
	if [ ! $? -eq 0 ]; then
		return 1
	fi
	
	return 0
}

copy_STARS_info()
{
	remotehost=$1
	# Copy the answer back
	scp -o ConnectTimeout=75 -o BatchMode=yes -P $IDA_PRO_SERVER_PORT $IDA_PRO_SERVER_USER@$remotehost:${directory}/a.ncexe.* .
	return $?
}

server_has_cached_info()
{
	remotehost=$1
	# Copy the answer back
	ssh -o ConnectTimeout=10 -o BatchMode=yes -p $IDA_PRO_SERVER_PORT $IDA_PRO_SERVER_USER@$remotehost ls -l ${directory}/a.ncexe.infoannot > tmp.$$
	if [ -f tmp.$$ ]; then
		grep a.ncexe.infoannot tmp.$$
		if [ ! $? -eq 0 ]; then
			rm tmp.$$
			return 1
		else
			return 0
		fi
	fi
	return 1
}

copy_and_verify_result()
{
	remotehost=$1
	annotfile=$2

	copy_STARS_info $remotehost

	lines=`cat $annotfile | wc -l`
	if [ $lines -lt 10 ]; then
		echo "Failed to produce a valid annotations file for $annotfile."
		return 1 
	fi

	return 0
}

run_remote_command()
{
	remotehost=$1
	shift

	echo "Remote analyze on host $remotehost"
	ssh -o ConnectTimeout=10 -o BatchMode=yes -p $IDA_PRO_SERVER_PORT $IDA_PRO_SERVER_USER@$remotehost mkdir -p ${directory}

	# Check to see if the Ida Pro Server is too busy now and wait if necc.
	if [ -n "$MAX_IDA_PROCESSES" ]; then
	    while [ `ssh -o ConnectTimeout=10 -o BatchMode=yes -p $IDA_PRO_SERVER_PORT $IDA_PRO_SERVER_USER@$remotehost pgrep idal|wc -l` -ge "$MAX_IDA_PROCESSES" ]; do
	        random=`od -An -N2 -tu2 /dev/urandom`
	        # Wait 10-30 seconds
		seconds=`expr $random % 20 + 10`
		echo Waiting $seconds seconds for an IDA process to exit...
		sleep $seconds
	    done
	fi

	scp -o ConnectTimeout=75 -o BatchMode=yes -P $IDA_PRO_SERVER_PORT -q $@ $IDA_PRO_SERVER_USER@$remotehost:$directory
	if [ ! $? -eq 0 ]; then
		return 1
	fi

	ssh -o ConnectTimeout=7200 -o BatchMode=yes -p $IDA_PRO_SERVER_PORT $IDA_PRO_SERVER_USER@$remotehost "cd ~/techx_umbrella/peasoup; source set_env_vars; export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:.; cd $directory; screen -D -L -ln -m -a -T xterm sh -x "'$SMPSA_HOME'"/SMP-analyze.sh $@" 2>&1
	if [ ! $? -eq 0 ]; then
		return 1
	fi

	copy_and_verify_result $remotehost $file.annot
	return $?
}

# FIXME: need to handle multiple files to be analyzed?

exit_code=1
verify_host_live $IDA_PRO_SERVER_HOST
if [  $? -eq 0 ]; then
	server_has_cached_info $IDA_PRO_SERVER_HOST
	if [ $? -eq 0 ]; then
		echo SERVER HAS ALREADY ANALYZED $md5name, retrieving cached info
		copy_and_verify_result $remotehost $file.annot
		exit_code=$?
	else
		run_remote_command $IDA_PRO_SERVER_HOST $@
		exit_code=$?
	fi
else
	echo "Host $IDA_PRO_SERVER_HOST is not responding"
fi

if [ ! $exit_code -eq 0 ]; then
	echo "Error detected on primary $IDA_PRO_SERVER_HOST, failing over to $IDA_PRO_SERVER_HOST2"

	verify_host_live $IDA_PRO_SERVER_HOST2
	if [ ! $? -eq 0 ]; then
		echo "Backup host $IDA_PRO_SERVER_HOST2 is not responding"
		exit 1
	fi

	run_remote_command $IDA_PRO_SERVER_HOST2 $@
	exit_code=$?
fi

exit $exit_code
